{*******************************************************}
{  HotImage Component Source                            }
{  Copyright (c) 1997,98 Miachin Alexander A.           }
{*******************************************************}

unit HotImage;
{$I HOTIMG.INC}

{  Version 2.2 , Delphi 4 adaptation
   Author: Miachin Alexander A. e-mail sasha@obladm.nso.ru}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;
{$I hotimg.inc}

type

  PPoints = ^TPoints;
  TPoints = array [0..(MaxListSize div 2) - 1] of TPoint;
  TScaleInfo = record
                  SrcSize:TSize;
                  SrcOffset:TPoint;
                  DstSize:TSize;
                  DstOffset:TPoint;
                end;

  TSpotKind = (skPolyline, skRectangle, skEllipse);

  TMaskKind = (mkMouseOver, mkMouseDown);
  TMaskKinds = set of TMaskKind;


  THotSpots = class;
  THotImage = class;

  {*** TMaskItem ***}
  TMaskPointArray = array[TMaskKind]of TPoint;

  PMaskItem = ^TMaskItem;
  TMaskItem = record
                 MaskKinds: TMaskKinds;
                 Offsets: TMaskPointArray;
              end;

  {*** TSpot ***}
  TSpot = class
  private
    FPoints: TList;
    FOwner: THotSpots;
    FPolyRgn: HRgn;
    FSpotKind: TSpotKind;
    FHintText: string;
    FID: Integer;
    FEnabled: Boolean;
    FMaskItem: PMaskItem;

    function GetRegion: HRgn;
    function CreateRegion: HRgn;
    function GetPoint(Index: Integer): Tpoint;
    procedure SetPoint(Index: Integer; const APoint: TPoint);
    procedure SetSpotKind(Value: TSpotKind);
    procedure CopyScaledPtArray(DstBuffer: PPoints; const ScaleInfo: TScaleInfo);

 //bounds manupulation
    function GetBoundsRect: TRect;
    procedure SetBoundsRect(const R: TRect);

    procedure AssignMasks(const Source: TMaskItem);
    function GetMaskItem: TMaskItem;
    function GetCount: Integer;
    procedure SetCount(Value: Integer);
  public
   //Constructors, destructors
    constructor Create(ASpotKind: TSpotKind);
    destructor Destroy; override;

    procedure Assign(Source: TSpot);

    //copy shape data with scaling
    procedure CopyShapeFrom(Source: TSpot; const ScaleInfo: TScaleInfo);

    //draw shape on destination canvas
    procedure DrawShape(DrawCanvas: TCanvas; const Offset: TPoint; AlignBrush: boolean);


    procedure ReleaseRegion;
    //returns pointer to array[0..n] of TPoint
    function GetPointArray: PPoints;
    //IS point in spot?
    function PointInSpot(const APoint: TPoint):boolean;
    //number of points in spot

    //Clear all points
    procedure Clear;
    //Bounds manipulation
    procedure Offset(dx, dy: Integer);
    function MaskPresent: Boolean;
    //Point manipulation
    procedure AddPoint(const APoint: TPoint);
    procedure DeletePoint(Index: Integer);
    procedure InsertPoint(Index: Integer; const APoint: TPoint);
    property Points[Index: Integer]: TPoint read GetPoint Write SetPoint; default;
    property Count: Integer read GetCount Write SetCount;
    property RgnHandle: HRgn read GetRegion;
    property HintText: string read FHintText write FHintText;
    property SpotID: Integer read FID write FID;
    property Enabled: Boolean read FEnabled write FEnabled;
    property SpotKind: TSpotKind read FSpotKind write SetSpotKind;
    property BoundsRect: TRect read GetBoundsRect write SetBoundsRect;
    property MaskItem: TMaskItem read GetMaskItem write AssignMasks;
  end;





  THotSpots = class(TPersistent)
  private
    FList: TList;
    FOwner: THotImage;
    function GetSpot(Index: Integer): TSpot;
    procedure SetSpot(Index: Integer; ASpot: TSpot);
    procedure WriteSpots(Stream: TStream);
    procedure ReadSpots(Stream: TStream);
    procedure WriteMaskItems(Stream: TStream);
    procedure ReadMaskItems(Stream: TStream);
    function MasksPresent: Boolean;
  protected
    procedure DefineProperties(Filer: TFiler); override;

  public
    constructor Create(AOwner: THotImage);
    destructor Destroy; override;

    procedure Assign(Source: TPersistent); override;


    {returns  rectangle, that bounds all spots}
    function GetBoundsRect: TRect;

    {Streaming routines}
    procedure SaveToStream(Stream: TStream);
    procedure LoadFromStream(Stream: TStream);

    {File routines}
    procedure SaveToFile(const FileName: string);
    procedure LoadFromFile(const FileName: string);

    procedure SaveAsText(const TextFileName: string);
    procedure LoadAsText(const TextFileName: string);


    {Spot manipulations}
    procedure DeleteSpot(Index: integer);
    function AddSpot(ASpot: TSpot): integer;
    function RemoveSpot(ASpot: TSpot): TSpot;
    procedure InsertSpot(Index: Integer; ASpot: TSpot);
    procedure MoveSpot(CurrentIndex, NewIndex: Integer);
    procedure Clear;

    function GetSpotByID(ASpotID: Integer): TSpot;
   //Coordinates routines
    function IndexByPoint(const Point: TPoint): integer;
    function SpotByPoint(const Point: TPoint): TSpot;
    function Count:Integer;

    property Spots[Index:Integer]:TSpot read GetSpot write SetSpot; default;
    property Owner:THotImage read FOwner;
  end;

  //Event for OnGetHintText
  TGetHintTextEvent=procedure (Sender: TObject; Spot: TSpot;
                                var Text: String) of object;

  TSpotEvent = procedure (Sender: TObject; Spot: TSpot) of object;

  THotImage = class(TImage)
  private
    { Private declarations }
    FHotSpots: THotSpots;
    FMaskPicture: TPicture;
    FPaintBmp: TBitMap;
    FPaintTopLeft: TPoint;{in source coordinates}
    FMaskValid: Boolean;
    FSelectedSpot: TSpot;
    FOldPoint: TSmallPoint;

    FCapturedSpot: TSpot;
    FDown: Boolean;

    FDragging: Boolean;
    FCommonHint: String;
    FOnPaint: TNotifyEvent;
    FOnGetHintText: TGetHintTextEvent;
    FOnSelectSpot: TSpotEvent;
    FHotAreas: Boolean;
    FHotCursor: TCursor;
    FCommonCursor: TCursor;

    procedure SetSpotPicture(Spot: TSpot; Down: Boolean);
    function  GetCanvas: TCanvas;
    function  GetPictureCanvas: TCanvas;
    procedure SetHotSpots(AHotSpots: THotSpots);

    {Added 26.02.98}
    procedure SetCommonHint(const Value: String);
    procedure SetCommonCursor(Value: TCursor);

    {Added 7.03.98}
    function GetSpotIndex: Integer;
    procedure SetSelectedSpot(Value: TSpot);
    procedure SetMaskPicture(Value: TPicture);

{$IFNDEF _D3_}
    function DestRect: TRect;
{$ELSE}
    function GetTransparent: Boolean;
    procedure SetTransparent(Value: Boolean);
{$ENDIF}
  protected
    { Protected declarations }
    procedure CMHintShow(var Message: TMessage); message CM_HINTSHOW;
    procedure CMHitTest(var Message: TCMHitTest); message CM_HITTEST;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
    procedure WMMouseMove(var Message: TWMMouseMove);message WM_MOUSEMOVE;

    procedure Click; override;
    procedure DblClick; override;
    procedure Paint; override;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function CopyPointArray(PointBuffer: PPoints; Spot: TSpot): boolean;
    function CreateRgnHandle(Spot: TSpot): HRgn;
    function CreateRgnByID(ASpotID: integer): HRgn;
    procedure ResetSelection;
    function ScaleInfo: TScaleInfo;
    function SpotFromPos(X, Y: Integer): TSpot;
    property SelectedSpot: TSpot read FSelectedSpot;
    property CapturedSpot: TSpot read FCapturedSpot;
    property SpotIndex: Integer read GetSpotIndex;
    property Canvas: TCanvas read GetCanvas;
    property PictureCanvas: TCanvas read GetPictureCanvas;
  published
    { Published declarations }
    property HotSpots: THotSpots read FHotSpots write SetHotSpots;
    property Hint read FCommonHint write SetCommonHint;     //hides original TControl.Hint
    property Cursor: TCursor read FCommonCursor write SetCommonCursor default crDefault;
    property HotCursor: TCursor read FHotCursor write FHotCursor default{$IFDEF __D3__} crHandPoint {$ELSE} crDefault{$ENDIF};
    property HotAreasOnly: Boolean read FHotAreas write FHotAreas default true;
    property MaskPicture: TPicture read FMaskPicture write SetMaskPicture;
{$IFDEF _D3_}
    property Transparent: Boolean read GetTransparent write SetTransparent default false;
{$ENDIF}
    {Events}
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    property OnGetHintText: TGetHintTextEvent read FOnGetHintText write FOnGetHintText;
    property OnSelectSpot: TSpotEvent read FOnSelectSpot write FOnSelectSpot;
  end;


implementation
uses hotparse;

{*** TSpot ***}

{modified 19.02.98}
constructor TSpot.Create(ASpotKind: TSpotKind);
begin
  inherited Create;
  FPoints := TList.Create;
  FEnabled := true;
  FSpotKind := ASpotKind;
  {all SpotKinds except skPolyline have fixed number of points}
  if ASpotKind <> skPolyLine then
  FPoints.Count := 4;
end;

destructor TSpot.Destroy;
begin
  ReleaseRegion;
  FPoints.Free;
  if assigned(FMaskItem) then Dispose(FMaskItem);
  if Assigned(FOwner) then FOwner.RemoveSpot(Self);
  inherited Destroy;
end;


procedure TSpot.Assign(Source:TSpot);
var
  cnt: integer;
begin
    ReleaseRegion;
    With Source.FPoints do
    begin
      cnt := Count;
      Self.FPoints.Count := cnt;
      system.Move(List[0], Self.FPoints.List[0], cnt * sizeof(pointer) );
    end;
    FSpotKind := Source.FSpotKind;
    HintText := Source.HintText;
    Enabled := Source.Enabled;
    SpotID := Source.SpotID;
end;


function TSpot.GetMaskItem:TMaskItem;
begin
  if assigned(FMaskItem) then
    Result := FMaskItem^
  else
    Fillchar(result, sizeof(TMaskItem), 0);
end;

procedure TSpot.AssignMasks(const Source:TMaskItem);
begin
  if Source.MaskKinds <> [] then
  begin
    if not Assigned(FMaskItem) then  New(FMaskItem);
    FMaskItem^ := Source;
  end
   else
   begin
     Dispose(FMaskItem);
     FMaskItem := nil;
   end;
end;


function TSpot.MaskPresent: Boolean;
begin
  Result := assigned(FMaskItem);
end;

procedure TSpot.CopyScaledPtArray(DstBuffer: PPoints; const ScaleInfo: TScaleInfo);
var
  i, Cnt: integer;
  SrcPts: PPoints;
begin
  Cnt := Count;
  SrcPts := GetPointArray;
  for i := 0 to Cnt - 1 do
  begin
    With DstBuffer^[i], ScaleInfo do
    begin
      x:=MulDiv(SrcPts^[i].x + SrcOffset.x, DstSize.cx, SrcSize.cx) + DstOffset.x;
      y:=MulDiv(SrcPts^[i].y + SrcOffset.y, DstSize.cy, SrcSize.cy) + DstOffset.y;
    end;
  end;
end;

procedure TSpot.CopyShapeFrom(Source: TSpot; const ScaleInfo: TScaleInfo);
begin
  ReleaseRegion;
  if assigned(Source) then
  begin
    FSpotKind := Source.FSpotKind;
    Self.FPoints.Count := Source.FPoints.Count;
    Source.CopyScaledPtArray(GetPointArray, ScaleInfo);
  end
   else Clear;
end;


procedure TSpot.DrawShape(DrawCanvas: TCanvas; const Offset: TPoint;
                          AlignBrush: Boolean);
var
  DC: HDC;
  tmpPt, Oldpos: TPoint;
  Points: PPoints;
  hBr: HBRUSH;
begin
  if FPoints.Count = 0 then exit;
  DC := DrawCanvas.Handle;
  OffsetViewPortOrgEx(DC, Offset.x, Offset.y, OldPos);
  if AlignBrush then
  begin
  {Remove old brush from device context}
    hBr := SelectObject(DC, GetStockObject(BLACK_BRUSH));
    UnrealizeObject(hBr);
    tmpPt := Point(0,0);
    LPtoDP(DC, tmpPt, 1);

    SetBrushOrgEx(DC, tmpPt.x mod 8, tmpPt.y mod 8,{$IFDEF _D3_}@{$ENDIF}tmpPt);
  {Return back again in device context}
    SelectObject(DC, hBr);
  end;
  Points:=GetPointArray;
  With DrawCanvas do
  case FSpotKind of
    skPolyLine: Polygon(Slice(Points^, Count));
    skRectangle: With PRect(Points)^ do Rectangle(Left,Top,Right,Bottom);
    skEllipse: With PRect(Points)^ do Ellipse(Left,Top,Right,Bottom);
  end;
  SetViewPortOrgEx(DC, OldPos.x, OldPos.y, nil);
  if AlignBrush then
    SetBrushOrgEx(DC,tmpPt.x, tmpPt.y,{$IFDEF _D3_}nil{$ELSE}tmppt{$ENDIF});
end;





procedure TSpot.ReleaseRegion;
begin
  If FPolyRgn <>0 then
  begin
    DeleteObject(FPolyRgn);
    FPolyRgn := 0;
  end;
end;

function TSpot.GetRegion:HRgn;
begin
  if FPolyRgn = 0 then
    FPolyRgn := CreateRegion;
  Result := FPolyRgn;
end;

function TSpot.CreateRegion:HRgn;
var
  tempRect: PRect;
begin
    if FSpotKind = skPolyline then
      Result := CreatePolygonRgn(GetPointArray^, Count, ALTERNATE)
    else
     begin
       TempRect:=PRect(GetPointArray);
       case FSpotKind of
        skRectangle: Result := CreateRectRgnIndirect(TempRect^);
        skEllipse: Result := CreateEllipticRgnIndirect(TempRect^);
       else
         Result := 0;
       end;
     end;
end;

function TSpot.GetCount:integer;
begin
  result := FPoints.Count div 2;
end;

procedure TSpot.SetCount(Value: Integer);
begin
  if Count<>Value then
  begin
    ReleaseRegion;
    FPoints.Count := Value * 2;
    SetSpotKind(skPolyLine);
  end;
end;

procedure TSpot.Clear;
begin
  ReleaseRegion;
  SetSpotKind(skPolyLine);
  FPoints.Clear;
end;

procedure TSpot.InsertPoint(Index: Integer; const APoint: TPoint);
begin
  Index := Index*2;
  ReleaseRegion;
  SetSpotKind(skPolyLine);
  FPoints.Insert(Index, Pointer(APoint.Y));
  FPoints.Insert(Index, Pointer(APoint.X));
end;

function TSpot.GetPoint(Index: Integer): TPoint;
begin
  Index:=Index*2;
  With Result,FPoints do
  begin
    x := longint(Items[Index]);
    y := longint(Items[Index+1]);
  end;
end;

{added 19.02.98}
procedure TSpot.SetSpotKind(Value: TSpotKind);
begin
  ReleaseRegion;
  if Value <> FSpotKind then
  begin
    if Value <> skPolyline then FPoints.Count := 4;
    FSpotKind := Value;
  end;
end;


procedure TSpot.SetPoint(Index: Integer; const APoint: TPoint);
begin
  Index := Index * 2;
  ReleaseRegion;
  FPoints.Items[Index] := Pointer(APoint.x);
  FPoints.Items[Index+1] := Pointer(APoint.y);
end;

procedure TSpot.AddPoint(const APoint: TPoint);
begin
  SetSpotKind(skPolyLine);
  FPoints.Add(Pointer(APoint.X));
  FPoints.Add(Pointer(APoint.Y));
end;


procedure TSpot.DeletePoint(Index: Integer);
begin
  Index := Index * 2;
  SetSpotKind(skPolyLine);
  FPoints.Delete(Index);
  FPoints.Delete(Index);
end;

function TSpot.GetPointArray: PPoints;
begin
  Result := PPoints(@FPoints.List[0]);
end;

function TSpot.PointInSpot(const APoint: TPoint): Boolean;
begin
{if spotkind have rect area - just call PtInRect(without creating regions)}
  if FSpotkind = skRectangle then
     Result := PtInRect(PRect(GetPointArray)^, APoint)
  else
   begin
     with APoint do Result := PtInRegion(GetRegion, X, Y);
     if not Result then ReleaseRegion;
   end;
end;


{bounds manupulation}
function TSpot.GetBoundsRect: TRect;
begin
  if FSpotKind = skPolyline then
  begin
    if GetRgnBox(GetRegion, Result) = 0 then
      FillChar(Result, sizeof(Result), 0)
      else
      begin
       {GetRgnBox returns rectangle,
        without 1 right and 1 bottom point}
       inc(Result.Right);
       inc(Result.Bottom);
      end;
  end
   else result:=PRect(GetPointArray)^;

end;

procedure TSpot.SetBoundsRect(const R: TRect);
var
  OldRect: TRect;
  ptCount: Integer;
  P: PPoint;
  Nom, Denom: TPoint;
begin
  ReleaseRegion;
  if FSpotKind = skPolyline then
  begin
    OldRect := GetBoundsRect;
    Nom.X := R.Right - R.Left;
    Nom.Y := R.Bottom - R.Top;
    Denom.X := OldRect.Right - OldRect.Left;
    Denom.Y := OldRect.Bottom - OldRect.Top;
    ptCount := Count;
    p := PPoint(GetPointArray);
    while ptCount > 0 do
    begin
      With P^ do
      begin
        x := R.Left + Muldiv(x - OldRect.Left, Nom.x, Denom.x);
        y := R.Top + Muldiv(y - OldRect.Top, Nom.y, Denom.y);
      end;
      inc(p);
      dec(ptCount);
    end;
  end
   else
     PRect(GetPointArray)^ := R;
end;

procedure TSpot.Offset(dx, dy: Integer);
var
  p:PPoint;
  ptCount:integer;
begin
  ReleaseRegion;
  ptCount := Count;
  p := PPoint(GetPointArray);
  while ptCount>0 do
  begin
    with p^ do
    begin
      inc(x, dx);
      inc(y, dy);
    end;
    inc(p);
    dec(ptCount);
  end;
end;


{*** THotSpots ***}
constructor THotSpots.Create(AOwner: THotImage);
begin
  inherited Create;
  FOwner := AOwner;
  FList := TList.Create;
end;

destructor THotSpots.Destroy;
begin
  Clear;
  FList.Free;
  inherited Destroy;
end;




procedure THotSpots.Clear;
begin
  While FList.Count > 0 do
     TSpot(FList.Last).Free;
end;


function THotSpots.GetSpot(Index:integer):TSpot;
begin
  Result:=TSpot(FList.Items[Index]);
end;

procedure THotSpots.SetSpot(Index: Integer; ASpot: TSpot);
begin
  TSpot(FList.Items[Index]).Assign(ASpot);
end;


procedure THotSpots.WriteSpots(Stream: TStream);
var
  Spot: TSpot;

  procedure WritePoints;
  var
    N: integer;
  begin
    With Stream do
    begin
      N := Spot.Count;
      WriteBuffer(N, Sizeof(N));
      WriteBuffer(Spot.GetPointArray^, sizeof(TPoint) * N);
    end;
  end;

  procedure WriteRect;
  begin
    Stream.WriteBuffer(Spot.GetPointArray^, sizeof(TRect));
  end;

var
  I,N,tmp: integer;
  SK: TSpotKind;
begin
  With Stream do
  begin
    N := 0;//indicating that new stream version loaded
    WriteBuffer(N, sizeof(N));
    N:=FList.Count;
    WriteBuffer(N, sizeof(N));
    for I := 0 to N-1 do
    begin
      Spot := Spots[I];
      SK := Spot.FSpotKind;
      Writebuffer(SK, 1); //only one byte needed
      WriteBuffer(Spot.FID, sizeof(Integer));
      WriteBuffer(Spot.FEnabled, sizeof(Boolean));
      tmp:=Length(Spot.FHintText);
      WriteBuffer(tmp, sizeof(Integer));
      if tmp<>0 then
        WriteBuffer(PChar(Spot.FHintText)^, tmp);

      if SK=skPolyline then
        WritePoints
       else
        WriteRect;
    end;
  end;
end;

procedure THotSpots.ReadSpots(Stream: TStream);
var
  Spot: TSpot;
  NewVersion : Boolean;

  procedure ReadPoints;
  var
    N: integer;
  begin
    With Stream do
    begin
      ReadBuffer(N, sizeof(N));
      Spot.FPoints.Count := N * 2;
      ReadBuffer(Spot.GetPointArray^, sizeof(TPoint) * N);
    end;
  end;

  procedure ReadRect;
  begin
    Stream.ReadBuffer(Spot.GetPointArray^, sizeof(TRect));
  end;

var
  I, N, tmp: Integer;
  SK: TSpotKind;
begin
  Clear;
  With Stream do
  begin
    ReadBuffer(N, sizeof(N));
    NewVersion := N = 0;
    if NewVersion then ReadBuffer(N, sizeof(N));
    For I := 0 to  N-1 do
    begin
      SK := skPolyline;
      if NewVersion then ReadBuffer(SK, 1);{one byte}
      Spot := TSpot.Create(SK);
      AddSpot(Spot);
      ReadBuffer(Spot.FID, sizeof(Integer));
      ReadBuffer(Spot.FEnabled, sizeof(Boolean));
      ReadBuffer(tmp, sizeof(Integer));
      SetLength(Spot.FHintText, tmp);
      if tmp <> 0 then
        ReadBuffer(PChar(Spot.FHintText)^, tmp);
      if sK = skPolyline then
        ReadPoints
       else
        ReadRect;
    end;
  end;
end;

procedure THotSpots.WriteMaskItems(Stream: TStream);

  procedure WriteMask(const MaskItem: TMaskItem; Index: Integer);
  begin
    With Stream do
    begin
      WriteBuffer(Index, sizeof(Integer));
      if Index >= 0 then
      begin
        WriteBuffer(MaskItem.MaskKinds, 1);
        WriteBuffer(MaskItem.Offsets, sizeof(TMaskPointArray));
      end;
    end;
  end;

  procedure GetMaskList(List: TList);
  var
    I, N: integer;
  begin
    N := FList.Count;
    for I := 0  to  N-1 do
    begin
     with TSpot(FList.List[i]) do
      if Assigned(FMaskItem) then List.Add(pointer(i));
    end;
  end;

var
  N, I, Index: Integer;
  MaskList: TList;

begin
  N := sizeof(TMaskPointArray);{Offset block size}
  Stream.WriteBuffer(N, sizeof(Integer));
  MaskList := TList.Create;
  try
    GetMaskList(MaskList);
    N := MaskList.Count;{Number of items}
    Stream.WriteBuffer(N, sizeof(Integer));

    for i := 0 to  N-1 do
    begin
      Index := Longint(MaskList.List[i]);
      WriteMask(TSpot(FList.Items[Index]).FMaskItem^, Index);
    end;

  finally
    MaskList.Free;
  end;
end;


procedure THotSpots.ReadMaskItems(Stream: TStream);
var
  BytesToRead,
  BytesToSkip: Integer;
  MaskItem: TMaskItem;

 procedure ReadMask;
  const
      MaskItemMask: TMaskKinds = [Low(TMaskKind)..High(TMaskKind)];
  var
    SpotIndex:integer;
 begin
   With Stream do
   begin
     FillChar(MaskItem, sizeof(TMaskItem), 0);
     ReadBuffer(SpotIndex, sizeof(Integer));

     if SpotIndex < 0 then exit;

     ReadBuffer(MaskItem.MaskKinds, 1);
     MaskItem.MaskKinds := MaskItem.MaskKinds*MaskItemMask;
     ReadBuffer(MaskItem.Offsets, BytesToRead);
     if BytesToSkip > 0 then
       Seek(BytesToSkip, soFromCurrent);
     if (SpotIndex < FList.Count) then
       TSpot(FList[SpotIndex]).AssignMasks(MaskItem);
   end;
 end;


var
  N, I:integer;
begin
  Stream.ReadBuffer(N, sizeof(Integer));{size of offset points block}
  if N > sizeof(TMaskPointArray) then
  begin
    BytesToSkip := N - sizeof(TMaskPointArray);
    BytesToRead := sizeof(TMaskPointArray);
  end
   else
   begin
    BytesToSkip := 0;
    BytesToRead := N;
   end;
  Stream.ReadBuffer(N, sizeof(Integer));{number of entries}
  for i := 0 to N-1 do ReadMask;
end;

function THotSpots.MasksPresent: Boolean;
var
   I:Integer;
begin
   Result:=false;
   for I:=0 to FList.Count - 1 do
   begin
    Result:=Assigned(TSpot(FList.List[i]).FMaskItem);
    if Result then break;
   end;
end;

procedure THotSpots.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineBinaryProperty('Spots', ReadSpots, WriteSpots, FList.Count > 0);
  Filer.DefineBinaryProperty('MaskItems', ReadMaskItems, WriteMaskItems, MasksPresent);
end;



procedure THotSpots.Assign(Source: TPersistent);
var
  i, Cnt: integer;
  ASpot: TSpot;
  MiPtr: PMaskItem;
begin
  If (Source <> nil) and (Source is THotSpots) then
  begin
    Clear;
    Cnt := THotSpots(Source).FList.Count - 1;
    For i := 0 to Cnt do
    begin
      ASpot := TSpot.Create(skPolyLine);
      AddSpot(ASpot);
      ASpot.Assign(THotSpots(Source).Spots[i]);
      MiPtr := THotSpots(Source).Spots[i].FMaskItem;
      if Assigned(MiPtr)then
        ASpot.AssignMasks(MiPtr^);
    end;
  end
   else
    inherited Assign(Source);
end;


procedure THotSpots.DeleteSpot(Index: Integer);
begin
  TSpot(FList[Index]).Free;
end;


function THotSpots.AddSpot(ASpot:TSpot):integer;
begin
  Result:=FList.Add(ASpot);
  ASpot.FOwner:=Self;
end;

function THotSpots.RemoveSpot(ASpot:TSpot):TSpot;
begin
  Result := ASpot;
  if Assigned(FOwner) and (FOwner.FSelectedSpot = ASpot) then
    FOwner.SetSelectedSpot(nil);
  FList.Remove(ASpot);
  With ASpot do
     FOwner := nil;
end;

procedure THotSpots.MoveSpot(CurrentIndex,NewIndex:integer);
begin
  FList.Move(CurrentIndex, NewIndex);
end;

procedure THotSpots.InsertSpot(Index: Integer; ASpot: TSpot);
begin
  FList.Insert(Index, ASpot);
  ASpot.FOwner := Self;
end;




function THotSpots.IndexByPoint(const Point: TPoint): Integer;
var
  I, Cnt: integer;
begin
  Result := -1;
  Cnt := FList.Count - 1;
  for I := 0 to Cnt do
  with Spots[I]do
    if Enabled and PointInSpot(Point) then
    begin
      Result := I;
      break;
    end;
end;

function THotSpots.GetSpotByID( ASpotID: Integer): TSpot;
var
  I:Integer;
  Spot:TSpot;
  Cnt:integer;
begin
  Result:=nil;
  Cnt:=FList.Count-1;
  for I:=0 to Cnt do
  begin
    Spot := TSpot(FList.Items[i]);
    if Spot.FID =ASpotID then
    begin
      Result:=Spot;
      break;
    end;
  end;
end;


function THotSpots.SpotByPoint(const Point: TPoint): TSpot;
var
  I:Integer;
begin
  I := IndexByPoint(Point);
  if I >= 0 then
    Result := Spots[I]
  else
    Result := nil;
end;

function THotSpots.Count: Integer;
begin
  Result := FList.Count;
end;






{streaming functions}
const HS_Sign: array [1..4] of char = 'HMAP';
      HS_Sign_20: array [1..4] of char = 'HMAp';
      MASK_SIGN: array[1..4] of char = 'MASK';

procedure THotSpots.SaveToStream(Stream: TStream);
begin
  Stream.WriteBuffer(HS_Sign,4);
  WriteSpots(Stream);
  Stream.WriteBuffer(MASK_Sign,4);
  WriteMaskItems(Stream);
end;



procedure THotSpots.LoadFromStream(Stream: TStream);

  procedure ReadSign;
  var
    I:integer;
  begin
    Stream.ReadBuffer(I,4);
    if (Longint(HS_Sign) <> I) and (Longint(HS_Sign_20) <> I) then
      raise EReadError.Create('HotMap: Invalid Stream format');
  end;

var
  MaskSign: Integer;
begin
  ReadSign;
  ReadSpots(Stream);
  if (Stream.Read(MaskSign,4) = 4) and (MaskSign = Longint(MASK_SIGN))then
    ReadMaskItems(Stream);
end;


procedure THotSpots.LoadAsText(const TextFileName: string);
begin
  ReadHSfromFile(TextFileName, Self);
end;


procedure THotSpots.SaveAsText(const TextFileName: string);
begin
  WriteHStoFile(TextFileName, Self)
end;


procedure THotSpots.LoadFromFile(const FileName: string);
var
  FS: TFileStream;
begin
  FS := TFileStream.Create(FileName, fmOpenRead or fmShareDenyRead);
  try
    LoadFromStream(FS);
  finally
    FS.Free;
  end;
end;

procedure THotSpots.SaveToFile(const FileName: string);
var
  FS: TFileStream;
begin
 FS := TFileStream.Create(FileName, fmCreate);
 try
   SaveToStream(FS);
 finally
   FS.Free;
 end;

end;



function THotSpots.GetBoundsRect:TRect;
var
  P: PPoint;

  procedure AdjustRect;
  begin
    with Result, P^ do
    begin
      if Left  > X then Left := X;
      if Right < X then Right := X;
      if Top   > Y then Top := Y;
      if Bottom< Y then Bottom := Y;

    end;
   
  end;

var
  Spot: TSpot;
  PointCount: Integer;
  I,J: Integer;

begin
  FillChar(Result, sizeof(TRect), 0);
  if FList.Count > 0 then
  begin
    I := 0;
    repeat
      Spot := Spots[I];
      PointCount := Spot.Count;
      inc(I);
    until (PointCount <> 0) or (I >= FList.Count);
    if I >= FList.Count then exit;
    P := PPoint(Spot.GetPointArray);
    Result.TopLeft := P^;
    Result.BottomRight := P^;
    For J:=1 to PointCount - 1 do
    begin
      inc(P);
      AdjustRect;
    end;
    while I < FList.Count do
    begin
      Spot:=Spots[I];
      PointCount:=Spot.Count;
      P:=PPoint(Spot.GetPointArray);
      for J:=0 to PointCount - 1 do
      begin
        AdjustRect;
        inc(P);
      end;
      inc(I);
    end;
  end; // if FList.Count>0
end;





{*** THotImage ***}
constructor THotImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMaskPicture := TPicture.Create;
 // ControlStyle := ControlStyle + [csOpaque];{doesnt require repaint background}
  FHotSpots := THotSpots.Create(Self);
  FHotAreas := True;
  FCommonCursor := crDefault;
{$IFDEF _D3_}
{"hand" cursor was defined since Delphi 3's VCL}
  FHotCursor := crHandPoint;
{$ELSE}
  FHotCursor := crDefault;
{$ENDIF}
end;

destructor THotImage.Destroy;
begin
  FPaintBmp.Free;
  FHotSpots.Free;
  FMaskPicture.Free;
  inherited Destroy;
end;



procedure THotImage.SetMaskPicture(Value: TPicture);
begin
  FMaskPicture.Assign(Value);
end;

procedure THotImage.SetHotSpots(AHotSpots: THotSpots);
begin
  FHotSpots.Assign(AHotSpots);
end;


procedure THotImage.SetCommonHint(const Value: string);
begin
  FCommonHint := Value;
  if FSelectedSpot = nil then
    inherited Hint := Value;
end;


procedure THotImage.SetCommonCursor(Value: TCursor);
begin
  FCommonCursor := Value;
  if FSelectedSpot = nil then
    inherited Cursor := Value;
end;



function THotImage.CreateRgnHandle(Spot: TSpot): HRgn;
var
  Buffer: PPoints;
  BufferSize: Integer;
  sk: TSpotKind;
  tempRect: PRect;

begin
  Result := 0;
  if Spot = nil then exit;

  BufferSize := Spot.Count * sizeof(TPoint);
  if BufferSize > 0 then
  begin
    sk := Spot.FSpotKind;
    GetMem(Buffer, BufferSize);
    try
      if CopyPointArray(Buffer, Spot) then
      begin
        if sk = skPolyline then
          Result:=CreatePolygonRgn(Buffer^, Spot.Count, ALTERNATE)
         else
         begin
           SK := Spot.FSpotKind;
           TempRect := PRect(Buffer);
           with TempRect^ do
             case SK of
              skRectangle: Result := CreateRectRgn(Left,Top,Right,Bottom);
              skEllipse: Result := CreateEllipticRgn(Left,Top,Right,Bottom);
             end;
           end;
         end;
    finally
      FreeMem(Buffer,BufferSize);
    end;
  end;//if BufferSize>0
end;

function THotImage.CreateRgnByID(ASpotID: Integer): HRgn;
var
  I: Integer;
  Spot: TSpot;
  TmpRgn: HRgn;

begin
  Result := 0;
  For  I := 0  to  FHotSpots.Count - 1 do
  begin
    Spot := FHotSpots[I];
    if Spot.SpotID = ASpotID then
    begin
      TmpRgn := CreateRgnHandle(Spot);
      if result = 0 then
         result := TmpRgn
       else
        begin
          CombineRgn(result,result,TmpRgn,RGN_OR);
          DeleteObject(TmpRgn);
        end;
    end;
  end;
end;

{$IFNDEF _D3_}
function THotImage.DestRect: TRect;
begin
 if Stretch then
    Result := ClientRect
  else if Center then
    Result := Bounds((Width - Picture.Width) div 2, (Height - Picture.Height) div 2,
      Picture.Width, Picture.Height)
  else
    Result := Rect(0, 0, Picture.Width, Picture.Height);
end;
{$ELSE}

function THotImage.GetTransparent: Boolean;
begin
  Result:=inherited Transparent;
end;

procedure THotImage.SetTransparent(Value: Boolean);
var G:TGraphic;
begin
  inherited Transparent := Value;
  G := FMaskPicture.Graphic;
  if (G <> nil) and (not ((G is TMetaFile) or (G is TIcon))) then
      G.Transparent := Value;
end;

{$ENDIF}

function THotImage.ScaleInfo: TScaleInfo;
begin
  With Result, Picture do
  begin
    SrcSize.cx := Width;
    SrcSize.cy := Height;
    Fillchar(SrcOffset, sizeof(TPoint), 0);
  end;
  With Result, DestRect do
  begin
    DstSize.cx := Right - Left;
    DstSize.cy := Bottom - Top;
    DstOffset := TopLeft;
  end;
end;

function THotImage.CopyPointArray(PointBuffer: PPoints; Spot: TSpot): Boolean;
begin
  Result := assigned(Spot)and assigned(PointBuffer);
  if Result then
    Spot.CopyScaledPtArray(PointBuffer, ScaleInfo);
end;



procedure THotImage.CMHintShow(var Message: TMessage);
{$IFNDEF _D3_}
{GetCursorHeightMargin from VCL in D3}
  { Return number of scanlines between the scanline containing cursor hotspot
    and the last scanline included in the cursor mask. }
  function GetCursorHeightMargin: Integer;
  var
    IconInfo: TIconInfo;
    BitmapInfoSize: Integer;
    BitmapBitsSize: Integer;
    Bitmap: PBitmapInfoHeader;
    Bits: Pointer;
    BytesPerScanline, ImageSize: Integer;

      function FindScanline(Source: Pointer; MaxLen: Cardinal;
        Value: Cardinal): Cardinal; assembler;
      asm
              PUSH    ECX
              MOV     ECX,EDX
              MOV     EDX,EDI
              MOV     EDI,EAX
              POP     EAX
              REPE    SCASB
              MOV     EAX,ECX
              MOV     EDI,EDX
      end;

  begin
    { Default value is entire icon height }
    Result := GetSystemMetrics(SM_CYCURSOR);
    if GetIconInfo(GetCursor, IconInfo) then
    try
      GetDIBSizes(IconInfo.hbmMask, BitmapInfoSize, BitmapBitsSize);
      Bitmap := AllocMem(BitmapInfoSize + BitmapBitsSize);
      try
        Bits := Pointer(Longint(Bitmap) + BitmapInfoSize);
        if GetDIB(IconInfo.hbmMask, 0, Bitmap^, Bits^) and
          (Bitmap^.biBitCount = 1) then
        begin
          { Point Bits to the end of this bottom-up bitmap }
          with Bitmap^ do
          begin
            BytesPerScanline := ((biWidth * biBitCount + 31) and not 31) div 8;
            ImageSize := biWidth * BytesPerScanline;
            Bits := Pointer(Integer(Bits) + BitmapBitsSize - ImageSize);
            { Use the width to determine the height since another mask bitmap
              may immediately follow }
            Result := FindScanline(Bits, ImageSize, $FF);
            { In case the and mask is blank, look for an empty scanline in the
              xor mask. }
            if (Result = 0) and (biHeight >= 2 * biWidth) then
              Result := FindScanline(Pointer(Integer(Bits) - ImageSize),
                ImageSize, $00);
            Result := Result div BytesPerScanline;
          end;
          Dec(Result, IconInfo.yHotSpot);
        end;
      finally
        FreeMem(Bitmap, BitmapInfoSize + BitmapBitsSize);
      end;
    finally
      if IconInfo.hbmColor <> 0 then DeleteObject(IconInfo.hbmColor);
      if IconInfo.hbmMask <> 0 then DeleteObject(IconInfo.hbmMask);
    end;
  end;
{$ENDIF}

type PHintInfo = ^THintInfo;

var
  HintInfoPtr: PHintInfo;

begin

  HintInfoPtr := PHintInfo(Message.LParam);
         
{$IFDEF _D3_}
  if assigned(FSelectedSpot)then
  begin
    HintInfoPtr^.HintStr := FSelectedSpot.FHintText;
    if assigned(FOnGetHintText) then
       FOnGetHintText(Self, FSelectedSpot, HintInfoPtr^.HintStr);
  end
   else
     HintInfoPtr^.HintStr := FCommonHint;
{$ELSE}
  {aligning Hint position (bug/feature of VCL in Delphi 2.0)}
  With HintInfoPtr^ do
  begin
    GetCursorPos(HintPos);
    Inc(HintPos.y, GetCursorHeightMargin);
  end;
{$ENDIF}
  Message.Result := 0;
end;



function THotImage.GetSpotIndex: Integer;
begin
  if FSelectedSpot = nil then Result := -1
   else Result := FHotSpots.FList.Indexof(FSelectedSpot);
end;

procedure THotImage.SetSelectedSpot(Value:TSpot);
{$IFNDEF _D3_}
var
  TempStr: string;

{$ENDIF}

begin
  if Value <> FSelectedSpot then
  begin

    FSelectedSpot := Value;
    Application.CancelHint;

    if Value <> nil then
    begin
{$IFNDEF _D3_}
      TempStr := Value.FHintText;
      {the only place to get hint text with VCL from Delphi 2.0}
      if assigned(FOnGetHintText)then
        FOnGetHintText(Self, Value, TempStr);
      inherited Hint := TempStr;

{$ENDIF}
      inherited Cursor := FHotCursor;
    end
     else
     begin
{$IFNDEF _D3_}
       inherited Hint := FCommonHint;
{$ENDIF}
       inherited Cursor := FCommonCursor;
     end;

     if Assigned(FOnSelectSpot) then FOnSelectSpot(Self, FSelectedSpot);
  end;
end;



{hack method to get access to protected property}
type THackClass = class(TGraphicControl);

function THotImage.GetCanvas: TCanvas;
begin
  Result := THackClass(Self).Canvas;
end;



function THotImage.GetPictureCanvas: TCanvas;
begin
  Result := inherited Canvas;
end;

{$WARNINGS OFF}
procedure THotImage.Paint;
var
  PaintDC: HDC;
  SavedIndex: Integer;
  MaskRect: TRect;
  PaintEvent: Boolean;
  SavedRgn: HRgn;
  SavedBox: TRect;
begin

  PaintEvent := assigned(FOnPaint);
  if FMaskValid then
  begin
    with FPaintBmp do
      MaskRect := Bounds(FPaintTopLeft.x, FPaintTopLeft.y, Width,height);
    PaintDC := Canvas.Handle;
    SavedIndex := SaveDC(PaintDC);
    try
      SetMapMode(PaintDC, MM_ANISOTROPIC);
      With ScaleInfo do
      begin
        SetWindowExtEx(PaintDC, SrcSize.cx, SrcSize.cy, nil);
        SetWindowOrgEx(PaintDC, SrcOffset.x, SrcOffset.y, nil);
        SetViewPortExtEx(PaintDC, DstSize.cx, DstSize.cy, nil);
        SetViewPortOrgEx(PaintDC, DstOffset.x + Left, DstOffset.y + Top, nil);
      end;
      LPtoDP(PaintDC, MaskRect, 2);
      Canvas.Draw(FPaintTopLeft.x, FPaintTopLeft.y, FPaintBmp);
    finally
      RestoreDC(PaintDC, SavedIndex);
    end;

    if PaintEvent then
       GetClipBox(PaintDC, SavedBox);

    DPtoLP(PaintDC, MaskRect, 2);
    With MaskRect do
      ExcludeClipRect(PaintDC, Left, Top, Right, Bottom);
  end;

  inherited Paint;

  if PaintEvent then
  begin
    if FMaskValid then
    begin
      SavedRgn := CreateRectRgnIndirect(SavedBox);
      SelectClipRgn(PaintDC, SavedRgn);
      DeleteObject(SavedRgn);
    end;
    FOnPaint(Self);
  end;

end;

{$WARNINGS ON}


function THotImage.SpotFromPos(X, Y: Integer): TSpot;
begin
{ScaleInfo.SrcOffset always zero}
  with ScaleInfo do
    result:=FHotSpots.SpotByPoint(
    Point(MulDiv(X-DstOffset.x,SrcSize.cx,DstSize.cx),
          MulDiv(Y-DstOffset.y,SrcSize.cy,DstSize.cy)));
end;

procedure THotImage.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 if Enabled=false then
 begin
   SetSpotPicture(nil, false);
   Update;
 end;
end;

procedure THotImage.CMMouseLeave(var Message: TMessage);
begin
 inherited;
 if not FDragging then
 begin
   SetSpotPicture(nil, false);
 end;
end;
procedure THotImage.WMLButtonDown(var Message: TWMLButtonDown);
begin
  inherited;
  FDragging := true;
  SetSpotPicture(FSelectedSpot, true);
end;

procedure THotImage.WMLButtonUp(var Message: TWMLButtonUp);
begin
  FDragging := false;
//  inc(longint(FOldPoint));
  SetSpotPicture(FSelectedSpot,false);
  inherited;
end;




procedure THotImage.WMMouseMove(var Message: TWMMouseMove);
begin

  if FDragging then
  begin
    With Message do
    if FindDragTarget(ClientToScreen(SmallPointToPoint(Pos)), false) <> Self then
         SetSelectedSpot(nil);
    SetSpotPicture(FCapturedSpot, FCapturedSpot = FSelectedSpot);
  end
   else
    SetSpotPicture(FSelectedSpot,false);
  inherited;

end;


{The main cause of having CM_HITTEST handler -
 its' the only place where  Cursor and HintText normaly can be handled}
procedure THotImage.CMHitTest(var Message: TCMHitTest);
var
  Spot: TSpot;
begin
 with Message do
 begin
  if (Longint(FOldPoint) <> Longint(Pos)) then
  begin
   FOldPoint := Pos;
   if FDragging and assigned(FCapturedSpot)then
   begin
     With ScaleInfo do
       if FCapturedSpot.PointInSpot(
          Point(MulDiv(XPos-DstOffset.x,SrcSize.cx,DstSize.cx),
          MulDiv(YPos-DstOffset.y,SrcSize.cy,DstSize.cy))) then
         Spot := FCapturedSpot
       else
         Spot:=SpotFromPos(XPos, YPos);
   end
   else
     Spot:=SpotFromPos(XPos, YPos);
   SetSelectedSpot(Spot);
  end;
   result := 1;
 end;
end;


procedure THotImage.Click;
begin
 if (not FHotAreas) or ((FSelectedSpot <> nil)
     and (FCapturedSpot = FSelectedSpot)) then
   inherited Click;
end;


procedure THotImage.DblClick;
begin
  if (not FHotAreas) or ((FSelectedSpot<>nil)
      and (FCapturedSpot = FSelectedSpot)) then
    inherited DblClick;
end;


procedure THotImage.SetSpotPicture(Spot: TSpot; Down: Boolean);

  function IsMaskPresent(var Offs: TPoint): Boolean;
  begin
    if (Spot <> nil) and (Spot.FMaskItem <> nil) then
    begin
      with Spot.FMaskItem^ do
      begin
        Result := TMaskKind(Down) in MaskKinds;
        if not Result then
        begin
          Result := (Down = true) and (mkMouseOver in MaskKinds);
          if Result then  Down := false;
        end;
        if Result then Offs := Offsets[TMaskKind(Down)];
      end;
    end
     else Result:=false;
  end;


var
  NewRgn: HRgn;
  DC: HDC;
  OldRect, NewRect: TRect;
  Offs: TPoint;
  Masked: Boolean;

begin
  Masked := (MaskPicture.Graphic <> nil) and isMaskPresent(Offs);

  if (Spot <> FCapturedSpot) or (Down <> FDown) then
  begin
    FDown := Down;
    FCapturedSpot := Spot;
    if assigned(FPaintBmp) then
         with FPaintBmp, FPaintTopLeft do
          OldRect := Bounds(x, y, Width, Height);

    if Masked  then
    begin
      NewRgn := Spot.CreateRegion;{inattached region}
      try
        GetRgnBox(NewRgn, NewRect);
        FPaintTopLeft := NewRect.TopLeft;
        if not assigned(FPaintBmp) then FPaintBmp := TBitmap.Create;
        With NewRect do
        begin
         {FPaintBmp holds update part of image}
          FPaintBmp.Width := Right - Left;
          FPaintBmp.Height := Bottom - Top;
        end;
       {Painting on temporary bitmap}
        With FPaintBmp.Canvas do
        begin
          DC := Handle;
          {Set ViewPort origin}
          SetViewportOrgEx(DC, -NewRect.Left, -NewRect.Top, nil);

{$IFDEF _D3_}
          if inherited Transparent then
          begin
            Brush.Color := Self.Color;
            FillRect(NewRect);
          end;
{$ENDIF}

          OffsetRgn(NewRgn, -NewRect.Left, -NewRect.Top);
          if Spot.FSpotKind <> skRectangle then
          begin
            ExtSelectClipRgn(DC, NewRgn, RGN_DIFF);
            Draw(0, 0, Picture.Graphic);
          end;
          SelectClipRgn(DC, NewRgn);
          Draw(NewRect.Left - Offs.x, NewRect.Top - Offs.y, MaskPicture.Graphic);
          SetViewPortOrgEx(DC, 0, 0, nil);
        end;
      finally
        DeleteObject(NewRgn);
      end;
    end   //if Masked
     else
     begin
       FPaintBmp.free;
       FPaintBmp := nil
     end;

    if (FMaskValid or Masked) and (Parent <> nil) and Parent.HandleAllocated then
    begin
      DC := GetDC(0);
      SetMapMode(DC, MM_ANISOTROPIC);
      With ScaleInfo do
      begin
         SetWindowExtEx(DC,SrcSize.cx,SrcSize.cy,nil);
       //  SetWindowOrgEx(DC,SrcOffset.x,SrcOffset.y,nil);
         SetViewportExtEx(DC,DstSize.cx,DstSize.cy,nil);
         SetViewPortOrgEx(DC,DstOffset.x+Left,DstOffset.y+Top,nil);
      end;
      if FMaskValid then
      begin
        LPtoDP(DC, OldRect, 2);
        InvalidateRect(Parent.Handle, @OldRect, false);
      end;
      if Masked then
      begin
        LPtoDP(DC, NewRect, 2);
        InvalidateRect(Parent.Handle, @NewRect, false);
      end;
      ReleaseDC(0,DC);
    end;
    FMaskValid:=Masked;
  end;// if (Spot<>FCapturedSpot) or (Down<>FDown)

end;

procedure THotImage.ResetSelection;
begin
 if not FDragging then
   SetSpotPicture(nil,false);
end;


end.
