{*******************************************************}
{  HotImage Component Source                            }
{  Copyright (c) 1997,98 Miachin Alexander A.           }
{*******************************************************}

unit imskedit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls,hotimage, Buttons;

type
  TfrmMaskEdit = class(TForm)
    GroupBox2: TGroupBox;
    ScrollBox2: TScrollBox;
    GroupBox1: TGroupBox;
    ScrollBox1: TScrollBox;
    PaintBox1: TPaintBox;
    rgZoom: TRadioGroup;
    btnOk: TButton;
    btnCancel: TButton;
    gboxActions: TGroupBox;
    cboxAction: TComboBox;
    PaintBoxMask: TPaintBox;
    SpeedButton1: TSpeedButton;
    sbClearActn: TSpeedButton;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure rgZoomClick(Sender: TObject);
    procedure PaintBoxMaskPaint(Sender: TObject);
    procedure PaintBoxMaskMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBoxMaskMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PaintBoxMaskMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure cboxActionClick(Sender: TObject);
    procedure sbClearActnClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  private
    { Private declarations }

    FMaskGraphic:TGraphic;
    FTargetGraphic:TGraphic;
    FMaskItem:TMaskItem;

    FMaskKind:TMaskKind;
    FCleared: Boolean;
    FTargetBmp: TBitmap;
    FDragging: Boolean;

    FInSpot: Boolean;
    FPointsCount: Integer;

    FSpotTopLeft: TPoint;
    {delta between topleft of Spot on screen and where lbutton was down}
    FDelta: TPoint;

    FDisplayOffset: TPoint;
    FDrawRect: TRect;

    FInternalSpot: TSpot;
    FTargetSpot: TSpot;
   // FPtBuffSize:cardinal;
    FNominator: Integer;
    FDenominator: Integer;
    FTagNom: Integer;
    FTagDenom: Integer;
    procedure DrawShape(DrawCanvas: TCanvas; const Offst: TPoint);
    procedure InitTargetShape(Graphic: TGraphic);
    procedure Rescale(Nominator, Denominator: integer);
    procedure RecalcPoints;
    procedure FocusShape;


    function ItoD(const Point: TPoint): TPoint;
    function DtoI(const Point: TPoint): TPoint;

    procedure WriteOffsetString;
  public
    { Public declarations }
    function Edit(var MaskItem : TMaskItem; Spot : TSpot; 
                   MaskGraphic, TargetGraphic: TGraphic): Boolean;

  end;

implementation

uses ImskTest;

{$R *.DFM}
{$R-}


procedure TfrmMaskEdit.FormCreate(Sender: TObject);
begin
  Screen.Cursors[crSize] := LoadCursor(0, pchar(IDC_SIZEALL));
  FInternalSpot := TSpot.Create(skPolyLine);
  FTargetbmp := TBitMap.Create;
  FTargetbmp.Width := PaintBox1.Width;
  FTargetbmp.Height := PaintBox1.Height;
end;

procedure TfrmMaskEdit.FormDestroy(Sender: TObject);
begin
  FInternalSpot.Free;
  FTargetbmp.Free;
end;


procedure TfrmMaskEdit.WriteOffsetString;
begin
  label1.Update;
  if FCleared then Label1.Caption := ''
   else
     With FMaskItem.Offsets[FMaskKind] do
       Label1.Caption := Format('Offset: (%d,%d)',[x, y]);
end;


procedure TfrmMaskEdit.InitTargetShape(Graphic:TGraphic);
var
  SR: TRect;
  Offst: TPoint;
  DstSize: TPoint;
  SrcSize: TPoint;
  DCHnd: HDC;
begin
  With FTargetbmp.Canvas do
  begin
    Brush.Color := ScrollBox2.Color;
    FillRect(PaintBox1.ClientRect);
  end;
  SR := FTargetSpot.BoundsRect;
  FTargetSpot.ReleaseRegion;

  FSpotTopLeft := SR.TopLeft;
  DstSize.X := FTargetBmp.Width - 4;
  DstSize.Y := FTargetBmp.Height - 4;
  with SR do
  begin
    SrcSize.X := Right - Left;
    SrcSize.Y := Bottom - Top;
  end;
  DCHnd:=FTargetBmp.Canvas.Handle;

  SetMapMode(DCHnd, MM_ISOTROPIC);
  SetWindowExtEx(DCHnd, SrcSize.X, SrcSize.Y, nil);
  SetViewPortExtEx(DCHnd, DstSize.X, DstSize.Y, nil);

  Offst := DstSize;
  DPtoLP(DCHnd, Offst, 1);
  FTagNom := DstSize.Y;
  FTagDenom := Offst.Y;

  SetWindowOrgEx(DCHnd, SR.Left, SR.Top, nil);
  LPtoDP(DCHnd, SR.BottomRight, 1);

  if FTargetBmp.Width > SR.Right then
     DstSize.X := (FTargetBmp.Width - SR.Right) div 2
    else DstSize.X := 0;
   if FTargetBmp.Height > SR.Bottom then
     DstSize.Y := (FTargetBmp.Height - SR.Bottom) div 2
    else DstSize.Y := 0;
   {convert region data to device coordinates}
   LPtoDP(DCHnd, FInternalSpot.GetPointArray^, FPointsCount);
   OffsetViewPortOrgEx(DCHnd, DstSize.X, DstSize.Y, SrcSize);

  With FTargetBmp.Canvas do
   begin
    Draw(0,0,Graphic);
    SetMapMode(Handle, MM_TEXT);
    SetWindowOrgEx(Handle, 0, 0, nil);
   end;
   DrawShape(FTargetBmp.Canvas, Point(0,0));
end;




procedure TfrmMaskEdit.DrawShape(DrawCanvas: TCanvas; const Offst: TPoint);
begin
  With DrawCanvas do
  begin
    Pen.Color := clBlack;
    Pen.Width := 2;
    Pen.Style := psSolid;
    Pen.Mode := pmNotXor;
    Brush.Color := clBlack;
    Brush.Style := bsCross;
  end;
  FInternalSpot.DrawShape(DrawCanvas, Offst, true);
end;

function TfrmMaskEdit.ItoD(const Point: TPoint): TPoint;
begin
  Result.X:=MulDiv(Point.X, FNominator, FDenominator) + FDrawRect.Left;
  Result.Y:=MulDiv(Point.Y, FNominator, FDenominator) + FDrawRect.Top;
end;


function TfrmMaskEdit.DtoI(const Point:TPoint):TPoint;
begin
  Result.X := MulDiv(Point.X - FDrawRect.Left, FDenominator, FNominator);
  Result.Y := MulDiv(Point.Y - FDrawRect.Top, FDenominator, FNominator);
end;

procedure TfrmMaskEdit.FocusShape;
begin
  if FCleared then exit;
  With ScrollBox1 do
  begin
    VertScrollBar.Position := FDisplayOffset.y - 2;
    HorzScrollBar.Position := FDisplayOffset.x - 2;
  end;
end;

procedure TfrmMaskEdit.RecalcPoints;
var
  ScaleInfo: TScaleInfo;
begin
  With Scaleinfo do
  begin
    SrcSize.cx := FDenominator;
    SrcSize.cy := FDenominator;
    SrcOffset.x := - FSpotTopLeft.x;
    SrcOffset.y := - FSpotTopLeft.y;
    DstSize.cx := FNominator;
    DstSize.cy := FNominator;
    FillChar(DstOffset, sizeof(DstOffset), 0);
  end;
  FInternalSpot.CopyShapeFrom(FTargetSpot, ScaleInfo);
end;

procedure TfrmMaskEdit.Rescale(Nominator, Denominator:Integer);
var
  Ident: TPoint;
begin
  FNominator := Nominator;
  FDenominator := Denominator;


  PaintBoxMask.Visible := false;
  if assigned(FMaskGraphic) then
  begin
  FDrawRect:=Rect(0, 0, MulDiv(FMaskGraphic.Width, Nominator, Denominator),
                        MulDiv(FMaskGraphic.Height, Nominator, Denominator)
                  );
    if (FDrawRect.Right < ScrollBox1.ClientWidth) then
    begin
       Ident.X := (ScrollBox1.ClientWidth - FDrawRect.Right) div 2;
       PaintBoxMask.Width := ScrollBox1.ClientWidth;
    end
     else
     begin
       Ident.X := 0;
       PaintBoxMask.Width := FDrawRect.Right;
     end;
    if FDrawRect.Bottom < ScrollBox1.ClientHeight then
    begin
      Ident.Y := (ScrollBox1.ClientHeight-FDrawRect.Bottom)div 2;
      PaintBoxMask.Height := ScrollBox1.ClientHeight;
    end
     else
     begin
        Ident.Y :=0;
        PaintBoxMask.Height := FDrawRect.Bottom;
     end;
    OffsetRect(FDrawRect, ident.x, ident.y);
  end
   else
   begin
     FillChar(FDrawRect, sizeof(FDrawRect), 0);
     PaintBoxMask.Height := ScrollBox1.ClientHeight;
     PaintBoxMask.Width := ScrollBox1.ClientWidth;
   end;
  RecalcPoints;
  if not FCleared then
    FDisplayOffset := ItoD(FMaskItem.Offsets[FMaskKind]);

  PaintBoxMask.Left := 0;
  PaintBoxMask.Top := 0;
  PaintBoxMask.Visible := true;
  FocusShape;
end;



function TfrmMaskEdit.Edit(var MaskItem: TMaskItem;
                           Spot: TSpot;
                           MaskGraphic, TargetGraphic: TGraphic): Boolean;
begin
  FTargetSpot := Spot;
  FInternalSpot.Assign(Spot);
  FPointsCount := FInternalSpot.Count;
  FMaskItem := MaskItem;
  FMaskGraphic := MaskGraphic;
  FTargetGraphic := TargetGraphic;
  InitTargetShape(TargetGraphic);

  if FMaskItem.MaskKinds =[] then
    FMaskItem.MaskKinds := [mkMouseOver];
  if mkMouseOver in FMaskItem.MaskKinds then
    FMaskKind := mkMouseOver
   else
    FMaskKind := mkMouseDown;
  cBoxAction.ItemIndex := ord(FMaskKind);
  Rescale(FTagNom, FTagDenom);
  WriteOffsetString;
  Result := ShowModal = mrOk;
  if Result then MaskItem := FMaskItem;
end;


{painting target shape}
procedure TfrmMaskEdit.PaintBox1Paint(Sender: TObject);
begin
   PaintBox1.Canvas.Draw(0, 0, FTargetbmp);
end;

procedure TfrmMaskEdit.PaintBoxMaskPaint(Sender: TObject);
begin
  PaintBoxMask.Canvas.StretchDraw(FDrawRect, FMaskGraphic);
  if not FCleared then
    DrawShape(PaintBoxMask.Canvas, IToD(FMaskItem.Offsets[FMaskKind]));
end;


procedure TfrmMaskEdit.rgZoomClick(Sender: TObject);
var
  Nominator,
  Denominator: integer;
begin
  case rgZoom.ItemIndex of
   0:           {100%}
    begin
     Nominator := 1;
     Denominator := 1;
    end;
   1:              {200%}
    begin
     Nominator := 2;
     Denominator := 1;
    end;
   else           {Target}
    Nominator := FTagNom;
    Denominator := FTagDenom;
   end;

  Rescale(Nominator, Denominator);
  FocusShape;
end;


procedure TfrmMaskEdit.PaintBoxMaskMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and FInSpot then
  begin
    FDragging := true;
    Screen.Cursor := crSize;
  end;
end;

procedure TfrmMaskEdit.PaintBoxMaskMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  Offst: TPoint;
begin
  if not FDragging then
  begin
    FInSpot := false;
    if not FCleared then
    begin
      FDelta := ItoD(FMaskItem.Offsets[FMaskKind]);
      FDelta.X := X-FDelta.X;
      FDelta.Y := Y-FDelta.Y;
      FInSpot := FInternalSpot.PointInSpot(FDelta);
    end;
  end
   else
   begin
     PaintBoxMask.Update;
     Offst := DtoI(Point(X - FDelta.X, Y - FDelta.Y));
     With FMaskItem.Offsets[FMaskKind] do
       if (Offst.X <> X) or (Offst.Y <> Y) then
       begin
         DrawShape(PaintBoxMask.Canvas, FDisplayOffset);
         FMaskItem.Offsets[FMaskKind] := Offst;
         FDisplayOffset:=IToD(Offst);
         WriteOffsetString;
         DrawShape(PaintBoxMask.Canvas, IToD(FMaskItem.Offsets[FMaskKind]));
       end;
   end;
end;

procedure TfrmMaskEdit.PaintBoxMaskMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    FDragging := false;
    Screen.Cursor := crArrow;
  end;
end;



procedure TfrmMaskEdit.cboxActionClick(Sender: TObject);
var 
  MaskKind: TMaskKind;
begin
  MaskKind := TMaskKind(cboxAction.ItemIndex);
  with FMaskItem do
    if not(MaskKind in MaskKinds) then
    begin
       if MessageDlg('Make mask for new action?',
                      mtConfirmation,[mbYes,mbNo],0)<>mrYes then
       begin
         if FCleared then 
           cboxAction.ItemIndex := -1
          else
           cboxAction.ItemIndex := ord(FMaskKind);
         exit;
       end;
       MaskKinds := MaskKinds+[MaskKind];
       Offsets[MaskKind] := Point(0,0);
    end;
  if not FCleared then
  begin
    DrawShape(PaintBoxMask.Canvas,FDisplayOffset);
    FCleared := false;
  end;
  FMaskKind := MaskKind;
  FCleared := false;
  FDisplayOffset := IToD(FMaskItem.Offsets[MaskKind]);
  WriteOffsetString;
  DrawShape(PaintBoxMask.Canvas,FDisplayOffset);
  sbClearActn.Enabled := true;
  FocusShape;
end;

procedure TfrmMaskEdit.sbClearActnClick(Sender: TObject);
var
  MaskKinds: TMaskKinds;
  mk: TMaskKind;
begin
  DrawShape(PaintBoxMask.Canvas,FDisplayOffset);
  MaskKinds := FMaskItem.MaskKinds-[FMaskKind];
  FMaskItem.MaskKinds := MaskKinds;
  FCleared := true;
  for mk := Low(TMaskKind) to High(TMaskKind)do
    if mk in MaskKinds then
    begin
      FMaskKind := mk;
      FCleared := false;
      break;
    end;
  if not FCleared then
  begin
    cboxAction.ItemIndex := ord(FMaskKind);
    FDisplayOffset := IToD(FMaskItem.Offsets[FMaskKind]);
    DrawShape(PaintBoxMask.Canvas, FDisplayOffset);
  end
  else
   begin
     cboxAction.ItemIndex := -1;
     sbClearActn.Enabled := false;
   end;
  WriteOffsetString;
end;

procedure TfrmMaskEdit.SpeedButton1Click(Sender: TObject);
begin
  with TfrmTestMask.Create(nil)do
  try
    MakeTest(FTargetSpot, FMaskItem, FTargetGraphic, FMaskGraphic);
  finally
    Free;
  end;
end;

end.
